<?php
/**
 * import_wp_data.php
 * export_wp_data.phpでWordPressの記事や写真のデータをインポートするPHPスクリプトです。
 * このスクリプトは「Search-Replace-DB」を使ってDB内のドメイン文字列を置換し、そのドメインでアクセス出来るようになります。
 * https://interconnectit.com/search-and-replace-for-wordpress-databases/
 *
 * ■使い方
 * 本スクリプトをwp-load.phpと同ディレクトリに置くか、又は引数でwp-load.phpのパスを指定して実行します。
 * インポートするzipファイルのパス指定は必須です。
 *
 * ■引数
 * -z(必須) : export_wp_data.phpでエクスポートして、uploadsディレクトリに配置したzipのファイル名。
 * -w(オプション) : wp-load.phpのパス。ない場合は同ディレクトリにwp-load.phpがあるかをさがしにいきます。
 * -s(オプション) : コピー元のWordPressドメイン。
 * -r(オプション) : コピー先のWordPressドメイン。
 * -b(オプション) : バックアップフラグ。フラグがある場合はインポート前にデータをエクスポートする（要export_wp_data.php）
 * -e(オプション) : バックアップフラグがあった時のエクスポート先ディレクトリ。ない場合は同ディレクトリに出力します。
 *
 * ■セキュリティについて
 * 本スクリプトはCLIからしか実行出来ない最低限のセキュリティがありますが完璧ではありません。
 * ドキュメントルート外に置いて引数指定で実行して、役目を終えたらいずれにファイルもすぐに消すとより安全です。
 */

if (php_sapi_name() != 'cli') {
  die("only cli." . PHP_EOL);
}

echo_msg("start import_wp_data");

/**
 * パラメータの取得
 */
$options      = getopt("z:w::s::r::b::e::");
$zip_path     = (isset($options["z"])) ? $options["z"] : die("インポートzipファイル名指定は必須です。" . PHP_EOL);
$wp_load_path = (isset($options["w"])) ? $options["w"] : false; // 指定ない場合はこのディレクトリの想定
$source_str   = (isset($options["s"])) ? $options["s"] : false; // 置換元文字列
$replace_str  = (isset($options["r"])) ? $options["r"] : false; // 置換先文字列
$is_backup    = (array_key_exists("b", $options)) ? true : false;  // バックアップフラグはキーがあるかどうか
$export_dir   = (isset($options["e"])) ? $options["e"] : false; // 指定ない場合はこのディレクトリ

$zip_path = realpath($zip_path);
if (!file_exists($zip_path)) {
  die("インポートzipファイルが存在しません" . PHP_EOL);
}

// wp-load.phpファイルチェック
if ($wp_load_path) { // 指定があった時はパスチェック
  $wp_load_path = realpath($wp_load_path);  // 相対パスの場合は実行時の位置をもとに絶対パスにする
  if (!$wp_load_path) {
    die("パラメータ指定のwp-load.phpが存在しません" . PHP_EOL);
  }
} else {
  $wp_load_path = realpath(__DIR__ . "/wp-load.php"); // 指定ない場合このスクリプトのディレクトリを探す
  if (!$wp_load_path) {
    die("wp-load.phpが存在するディレクトリではありません" . PHP_EOL);
  }
}

// wp-load.phpを読み込めばDBの接続情報も読み込める
define('WP_DEBUG', false);  // エクスポート時にちょっとしたWPデバッグ情報は不要なのでfalseにする
require_once $wp_load_path;

// WPはUTC標準なのでここでは日本時間とする
date_default_timezone_set('Asia/Tokyo');

/* 定数を文字列内で扱いやすいように変数化する */
$host = DB_HOST;
$user = DB_USER;
$pass = DB_PASSWORD;
$db   = DB_NAME;

// DB接続確認
try {
  new PDO('mysql:host=' . $host . ';dbname=' . $db, $user, $pass);
} catch (PDOException $e) {
  die("MySQLに接続出来ません" . PHP_EOL);
}

/* バックアップ指定がある時はインポート前に現状のデータをバックアップする */
if ($is_backup) {
  // バックアップ有の時で引数でエクスポートディレクトリの指定がある場合はそこにzipを出力する
  if ($export_dir) { // 指定があった時はパスチェック
    $export_dir = realpath($export_dir);  // 絶対パスにする
    if (!$export_dir) {
      die("指定のエクスポートディレクトリが存在しません" . PHP_EOL);
    }
  } else {
    // 引数でエクスポートディレクトリの指定がない場合は本スクリプトと同ディレクトリにする
    $export_dir = __DIR__;
  }

  if (!file_exists(__DIR__ . "/export_wp_data.php")) {
    die("バックアップオプションには同ディレクトリにexport_wp_data.phpが必要です" . PHP_EOL);
  }
  chdir(__DIR__);
  // インポート時のバックアップでは現在のディレクトリに置く。バックアップ先を指定するなら-eパラメータで可能
  exec_cmd("php export_wp_data.php -w=\"{$wp_load_path}\" -e=\"{$export_dir}\"");
}

/* 解凍先をwp-content直下にしたいので移動する */
echo_msg("> cd " . WP_CONTENT_DIR);
chdir(WP_CONTENT_DIR);

/* 旧uploadsディレクトリの削除する */
$old_uploads_dir = WP_CONTENT_DIR . "/uploads"; // 一応絶対パス
if (file_exists($old_uploads_dir)) {   // 旧uploadsがある場合は削除する
  exec_cmd("rm -rf {$old_uploads_dir}");
}

/* 新uploadsディレクトリを解凍する */
exec_cmd("unzip -q {$zip_path}");

/* 新uploadsディレクトリはこの実行ユーザーで解凍されるのでアクセス権は変えておく（これは要検討） */
exec_cmd("chmod -R 777 uploads");

/* DBデータのインポート */
exec_cmd("mysql -u {$user} -h {$host} -p{$pass} {$db} < wordpress_db.sql");

/* 解凍したsqlファイルは削除する */
exec_cmd("rm wordpress_db.sql");

/* スクリプトの位置に戻ってSRDBがある場合のみDB内の置換処理をする */
echo_msg("> cd " . __DIR__);
chdir(__DIR__);

$srdb_path = __DIR__ . "/Search-Replace-DB/srdb.cli.php";
if (file_exists($srdb_path) && $source_str && $replace_str) {
  echo_msg("replace database string.");
  exec_cmd("php {$srdb_path} -h='{$host}' -u='{$user}' -p='{$pass}' -n='{$db}' -s='{$source_str}' -r='{$replace_str}'");
} else {
  echo_msg("DB内の文字列置換は実行しません");
}

echo_msg("success : import {$zip_path}");

// 最後に改行つけるだけのecho
function echo_msg($msg) {
  echo $msg . PHP_EOL;
}

// コマンド実行関数
function exec_cmd($cmd) {
  echo_msg("> {$cmd}");
  system($cmd, $return);  // exec()ではなくsystem()を使っているのは実行結果を出力してくれるから
  return $return;
}